# Generated by cpp11: do not edit by hand

force_materialization <- function(x) {
  invisible(.Call(`_vroom_force_materialization`, x))
}

vroom_materialize <- function(x, replace) {
  .Call(`_vroom_vroom_materialize`, x, replace)
}

vroom_convert <- function(x) {
  .Call(`_vroom_vroom_convert`, x)
}

vroom_str_ <- function(x) {
  .Call(`_vroom_vroom_str_`, x)
}

gen_character_ <- function(n, min, max, values, seed, seed2) {
  .Call(`_vroom_gen_character_`, n, min, max, values, seed, seed2)
}

guess_type_ <- function(input, na, locale, guess_integer) {
  .Call(`_vroom_guess_type_`, input, na, locale, guess_integer)
}

convert_connection <- function(in_con, out_con, from, to) {
  .Call(`_vroom_convert_connection`, in_con, out_con, from, to)
}

vroom_ <- function(inputs, delim, quote, trim_ws, escape_double, escape_backslash, comment, skip_empty_rows, skip, n_max, progress, col_names, col_types, col_select, name_repair, id, na, locale, guess_max, num_threads, altrep) {
  .Call(`_vroom_vroom_`, inputs, delim, quote, trim_ws, escape_double, escape_backslash, comment, skip_empty_rows, skip, n_max, progress, col_names, col_types, col_select, name_repair, id, na, locale, guess_max, num_threads, altrep)
}

has_trailing_newline <- function(filename) {
  .Call(`_vroom_has_trailing_newline`, filename)
}

vroom_rle <- function(input) {
  .Call(`_vroom_vroom_rle`, input)
}

utctime_ <- function(year, month, day, hour, min, sec, psec) {
  .Call(`_vroom_utctime_`, year, month, day, hour, min, sec, psec)
}

vroom_errors_ <- function(errors) {
  .Call(`_vroom_vroom_errors_`, errors)
}

vroom_fwf_ <- function(inputs, col_starts, col_ends, trim_ws, col_names, col_types, col_select, name_repair, skip, comment, skip_empty_rows, n_max, id, na, locale, guess_max, num_threads, altrep, progress) {
  .Call(`_vroom_vroom_fwf_`, inputs, col_starts, col_ends, trim_ws, col_names, col_types, col_select, name_repair, skip, comment, skip_empty_rows, n_max, id, na, locale, guess_max, num_threads, altrep, progress)
}

whitespace_columns_ <- function(filename, skip, n, comment) {
  .Call(`_vroom_whitespace_columns_`, filename, skip, n, comment)
}

vroom_write_ <- function(input, filename, delim, eol, na_str, col_names, append, options, num_threads, progress, buf_lines) {
  invisible(.Call(`_vroom_vroom_write_`, input, filename, delim, eol, na_str, col_names, append, options, num_threads, progress, buf_lines))
}

vroom_write_connection_ <- function(input, con, delim, eol, na_str, col_names, options, num_threads, progress, buf_lines, is_stdout, append) {
  invisible(.Call(`_vroom_vroom_write_connection_`, input, con, delim, eol, na_str, col_names, options, num_threads, progress, buf_lines, is_stdout, append))
}

vroom_format_ <- function(input, delim, eol, na_str, col_names, append, options, num_threads, progress, buf_lines) {
  .Call(`_vroom_vroom_format_`, input, delim, eol, na_str, col_names, append, options, num_threads, progress, buf_lines)
}
