% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vroom-package.R
\docType{package}
\name{vroom-package}
\alias{vroom-package}
\title{vroom: Read and Write Rectangular Text Data Quickly}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The goal of 'vroom' is to read and write data (like 'csv', 'tsv' and 'fwf') quickly. When reading it uses a quick initial indexing step, then reads the values lazily , so only the data you actually use needs to be read. The writer formats the data in parallel and writes to disk asynchronously from formatting.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://vroom.tidyverse.org}
  \item \url{https://github.com/tidyverse/vroom}
  \item Report bugs at \url{https://github.com/tidyverse/vroom/issues}
}

}
\author{
\strong{Maintainer}: Jennifer Bryan \email{jenny@posit.co} (\href{https://orcid.org/0000-0002-6983-2759}{ORCID})

Authors:
\itemize{
  \item Jim Hester (\href{https://orcid.org/0000-0002-2739-7082}{ORCID})
  \item Hadley Wickham \email{hadley@posit.co} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})
}

Other contributors:
\itemize{
  \item Shelby Bearrows [contributor]
  \item https://github.com/mandreyel/ (mio library) [copyright holder]
  \item Jukka Jylänki (grisu3 implementation) [copyright holder]
  \item Mikkel Jørgensen (grisu3 implementation) [copyright holder]
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}
